<?php

// {{{ Thaana format conversions

/**
 * Provides a number of functions for conversion/transliteration of text between various Thaana representation formats.
 *
 * This class currently provides the following functions:
 * convertUtf8ToUnicodeIntegers() - Convert UTF-8 data to Unicode character integer representations
 * convertUtf8ToAscii() - Convert UTF-8 data to Ascii
 * convertEntitiesToUnicodeIntegers() - Convert HTML Unicode entitied string to Unicode Integer characters array
 * convertEntitiesToUtf8 - Convert HTML Unicode entities to UTF-8
 * convertEntitiesToAscii() - Convert HTML Unicode entities to Dhivehi Ascii equivalents
 * convertUnicodeIntegersToUtf8() - Convert Unicode Integer array to UTF
 * convertUnicodeIntegersToEntities() - Convert Unicode char integers to HTML entities
 * convertUnicodeIntegersToAscii() - Convert Unicode char integers to Ascii
 * convertAsciiToUtf8() - Convert Ascii Thaana to UTf-8
 * convertAsciiToUnicodeEntities() - Convert Ascii Thaana to Unicode HTML entities
 * convertAsciiToUnicodeIntegers() - Convert Ascii Thaana to an array of Unicode integers
 *
 * Usage:
 * $thaana = new Thaana_Conversions();
 * echo $thaana->convertEntitiesToAscii('&#1931;&#1960;&#1928;&#1964;&#1920;&#1960;');
 * echo $thaana->convertAsciiToUtf8('rWacje');
 *
 *
 * @author Jawish Hameed
 * @link http://www.jawish.org/
 * @copyright  2008 Jawish Hameed
 * @license http://www.opensource.org/licenses/mit-license.php MIT License
 * @version Release: 0.2
 * @updated 2 Feb 2008
 */
class Thaana_Conversions
{
	// {{{ properties
	
	/**
	 * Mapping for Ascii to Unicode
	 *
	 * @var array
	 *
	 * @access private
	 */
	private $mapAsciiToUnicode = array(	'h' => '1920', 'S' => '1921', 'n' => '1922', 'r' => '1923',
										'b' => '1924', 'L' => '1925', 'k' => '1926', 'a' => '1927', 
										'v' => '1928', 'm' => '1929', 'f' => '1930', 'd' => '1931', 
										't' => '1932', 'l' => '1933', 'g' => '1934', 'N' => '1935', 
										's' => '1936', 'D' => '1937', 'z' => '1938', 'T' => '1939', 
										'y' => '1940', 'p' => '1941', 'j' => '1942', 'C' => '1943', 
										'X' => '1944', 'H' => '1945', 'K' => '1946', 'J' => '1947', 
										'R' => '1948', 'x' => '1949', 'B' => '1950', 'F' => '1951', 
										'Y' => '1952', 'Z' => '1953', 'A' => '1954', 'G' => '1955', 
										'q' => '1956', 'V' => '1957', 'w' => '1958', 'W' => '1959', 
										'i' => '1960', 'I' => '1961', 'u' => '1962', 'U' => '1963', 
										'e' => '1964', 'E' => '1965', 'o' => '1966', 'O' => '1967', 
										'c' => '1968', ',' => '1548', ';' => '1563', '?' => '1567', 
										')' => '0041', '(' => '0040', 'Q' => '65010'
										);
	
	/**
	 * Mapping for Unicode to Ascii
	 *
	 * @var array
	 *
	 * @access private
	 */
	private $mapUnicodeToAscii;
	
	// }}}
	// {{{ Class constructor
	
	/**
	 * Initializes the class by preparing internal variables
	 *
	 */
	public function __construct()
	{
		// Prepare the Unicode to Ascii translation map
		$this->mapUnicodeToAscii = array_flip($this->mapAsciiToUnicode);
	}
	
	// }}}
	// {{{ convertAsciiToUnicodeIntegers()
	
	/**
	 * Convert Ascii Thaana to an array of Unicode integers
	 *
	 * @param	string	$text		String Ascii text to convert
	 * @return	array		Array Unicode integer representations
	 */
	public function convertAsciiToUnicodeIntegers($input)
	{
		$output = array();
		
		// Loop through the chars in the text string
		for ($i = 0; $i < strlen($input); $i++) {
			
			// Check if the Ascii to Unicode lookup table defines the char code
			if (isset($this->mapAsciiToUnicode[$input[$i]])) {
				// Definition for char exists:
				
				// Add the looked-up code value to the output array
				array_push($output, $this->mapAsciiToUnicode[$input[$i]]);
				
			} else {
				// No definition for char:
				
				// Add char directly to output array
				array_push($output, ord($input[$i]));
			}
		}
		
		// Return converted text
		return $output;
	}
	
	// }}}
	// {{{ convertAsciiToUnicodeEntities()
	
	/**
	 * Convert Ascii Thaana to Unicode HTML entities
	 *
	 * @param	string	$text		String Ascii text to convert
	 * @return	string		String converted HTML entities
	 */
	public function convertAsciiToUnicodeEntities($text)
	{
		// Convert Ascii thaana into Unicode Integers
		$text = $this->convertAsciiToUnicodeIntegers($text);
		
		// Convert the Unicode Integers to Unicode HTML entities
		return $this->convertUnicodeIntegersToEntities($text);
	}
	
	// }}}
	// {{{ convertAsciiToUtf8()
	
	/**
	 * Convert Ascii Thaana to UTf-8
	 *
	 * @param	string	$text		String Ascii text to convert
	 * @return	string		String converted UTF-8 data
	 */
	public function convertAsciiToUtf8($text)
	{
		// Convert Ascii thaana into Unicode Integers
		$text = $this->convertAsciiToUnicodeIntegers($text);
		
		// Convert the Unicode Integers to Unicode HTML entities
		return $this->convertUnicodeIntegersToUtf8($text);
	}
	
	// }}}
	// {{{ convertUnicodeIntegersToAscii()
	
	/**
	 * Convert Unicode char integers to Ascii
	 *
	 * @param	mixed	$input		Integer or integer array to convert
	 * @return	string				String converted Ascii output
	 */
	public function convertUnicodeIntegersToAscii($input)
	{
		$output = '';
		
		// Force input to array if not already array
		if (!is_array($input)) $input = array($input);
		
		// Loop through input Unicode Integers array
		foreach ($input as $key => $char) {
			
			// Check if Unicode to Ascii mapping exists for char
			if (isset($this->mapUnicodeToAscii[$char])) {
				// Char mapping exists:
				
				// Get the mapped ASCII for the char and append to output
				$output .= $this->mapUnicodeToAscii[$char];
				
			} else {
				// Char is not mapped:
				
				// Convert char to ASCII and append to output
				$output .= chr($char);
			}
		}
		
		return $output;
	}
	
	// }}}
	// {{{ convertUnicodeIntegersToEntities()
	
	/**
	 * Convert Unicode char integers to HTML entities
	 *
	 * @param	mixed	$input		Integer or integer array to convert
	 * @return	string				String converted HTML entitied output
	 */
	public function convertUnicodeIntegersToEntities($input)
	{
		$output = '';
		
		// Force input to array if not already array
		if (!is_array($input)) $input = array($input);
		
		// Ensure argument is an array
		if (is_array($input)) {
			
			// Loop through each character
			foreach ($input as $key => $char) {
				
				// Convert char to HTML entity if is in Unicode range, convert to ASCII otherwise
				$output .= ($char < 256) ? chr($char) : '&#' . $char . ';';
			}
		}
		
		// Return output
		return $output;
	}
	
	// }}}
	// {{{ convertUnicodeIntegersToUtf8()
	
	/**
	 * Convert Unicode Integer array to UTF
	 *
	 * Adapted from original code by Scott Reynen on http://www.randomchaos.com/
	 *
	 * @param array	$text	Array of integer Unicode char representations
	 * @return string		String converted text in UTF
	 */
	public function convertUnicodeIntegersToUtf8($input)
	{
		$output = '';
		
		// Force input to array if not already array
		if (!is_array($input)) $input = array($input);
		
		// Loop through the input unicode char integers array
		foreach ($input as $char) {
			
			// Check char byte length
			if ($char < 128) {
				// Char is 1 byte, i.e. standard ASCII:
				
				// Convert to char and add to output
				$output.= chr($char);
				
			} elseif ($char < 2048) {
				// Char is 2 bytes:
				
				// Convert to char and add to output
				$output.= chr(192 + (($char - ($char % 64)) / 64));
				$output.= chr(128 + ($char % 64));
				
			} else {
				// Char is 3 bytes:
				
				// Convert to char and add to output
				$output.= chr(224 + (($char - ($char % 4096)) / 4096));
				$output.= chr(128 + ((($char % 4096) - ($char % 64)) / 64));
				$output.= chr(128 + ($char % 64));
			}
		}
		
		// Return output
		return $output;
	}
	
	// }}}
	// {{{ convertEntitiesToAscii()
	
	/**
	 * Convert HTML Unicode entities to Dhivehi Ascii equivalents
	 *
	 * @param	string $text	String with HTML encoded Unicode in Thaana range
	 * @return	string			String with the converted Ascii output
	 */
	public function convertEntitiesToAscii($input)
	{
		// Replace Unicode entities with Ascii equivalents
		$input = preg_replace_callback('/&#([0-9]+);/U', array(&$this, 'getAsciiChar'), $input);
		
		// Fix the numerics and return final output
		return $this->reverseNumerics($input);
	}
	
	// }}}
	// {{{ convertEntitiesToUtf8()
	
	/**
	 * Convert HTML Unicode entities to UTF-8
	 *
	 * @param	string $text	String with HTML encoded Unicode in Thaana range
	 * @return	string			String with the converted Utf8 output
	 */
	public function convertEntitiesToUtf8($input)
	{
		// Replace numeric entities
		$input = preg_replace_callback('/(?:&#([0-9]+);)/U', array(&$this, 'getUtf8Char'), $input);
		
		// Return output
		return $input;
	}
	
	// }}}
	// {{{ convertEntitiesToUnicodeIntegers()
	
	/**
	 * Convert HTML Unicode entitied string to Unicode Integer characters array
	 *
	 * @param	string $text	String with HTML encoded Unicode in Thaana range
	 * @return	array			Array with the converted Unicode integer character representation
	 */
	public function convertEntitiesToUnicodeIntegers($input)
	{
		// Convert entities to utf8
		$input = $this->convertEntitiesToUtf8($input);
		
		// Convert utf8 to unicode integers
		$input = $this->convertUtf8ToUnicodeIntegers($input);
		
		// Return output
		return $input;
	}
	
	// }}}
	// {{{ convertUtf8ToAscii()
	
	/**
	 * Convert UTF-8 data to Ascii
	 *
	 * @param	string	$text		String UTF-8 text to convert
	 * @return	string				String output in Ascii
	 */
	public function convertUtf8ToAscii($text)
	{
		$text = $this->convertUtf8ToUnicodeIntegers($text);
		$text = $this->convertUnicodeIntegersToAscii($text);
		return strrev($text);
	}
	
	// }}}
	// {{{ convertUtf8ToUnicodeIntegers()
	
	/**
	 * Convert UTF-8 data to Unicode character integer representations
	 *
	 * Adapted from original code by Scott Reynen on http://www.randomchaos.com/
	 *
	 * @param	string	$text		String UTF-8 text to convert
	 * @return	array				Array Unicode integer representations
	 */
	public function convertUtf8ToUnicodeIntegers($input)
	{
		$output = array();
		$temp = array();
		$lookingFor = 1;
		
		for ($i = 0; $i < strlen($input); $i++) {
			
			// Get the char code for the current char
			$charcode = ord($input[$i]);
			
			// Check char value
			if ($charcode < 128) {
				// Char is 1 byte, i.e. standard ASCII:
				
				// Add char to output
				$output[] = $charcode;
				
			} else {
				// Char takes 2 or 3 bytes, i.e extended ASCII and Unicode:
				
				// Get expected length of char
				if (count($temp) == 0) $lookingFor = ($charcode < 224) ? 2 : 3;
				
				// Add char code to temp storage
				$temp[] = $charcode;
				
				// Check if current char code has reached it's required length
				if (count($temp) == $lookingFor) {
					// Required length reached:
					
					// Build multibyte char code
					if ($lookingFor == 3) {
						$number = (($temp[0] % 16) * 4096) + (($temp[1] % 64) * 64) + ($temp[2] % 64);
					} else {
						$number = (($temp[0] % 32) * 64) + ($temp[1] % 64);
					}
					
					// Add char to output
					$output[] = $number;
					
					// Reset char code store anc status
					$temp = array();
					$lookingFor = 1;
				}
			}
		}
		
		// Return output
		return $output;
	}

	// }}}
	// {{{ getAsciiChar()
	
	/**
	 *  Helper function to get the Unicode equivalent for a character in ASCII
	 *
	 * This is a helper function used by the convertUnicodeIntegersToAscii() function
	 *
	 * @param	string $key		Char in unicode
	 * @return	string			String with Unicode character code
	 *
	 * @access private
	 */
	private function getAsciiChar($key)
	{
		return ($this->mapUnicodeToAscii[$key[1]]) ? $this->mapUnicodeToAscii[$key[1]] : $key[1];
	}
	
	// }}}
	// {{{
	
	/**
	 * Return the UTF-8 char for a given Unicode char integer specification
	 *
	 * This is a helper function used by the convertEntitiesToUtf8() function
	 *
	 * @param	integer	$code	Integer char code
	 * @return	string			String UTF-8 char
	 *
	 * @access private
	 */
	private function getUtf8Char($code)
	{
		return $this->convertUnicodeIntegersToUtf8($code[1]);
	}
	// }}}
	// {{{ reverseNumerics()
	
	/**
	 * Fix the numeric display order in the text from right-to-left to left-to-right
	 *
	 * @param	string	$text	String with the text to fix numerics
	 * @return	string			String with the numerics fixed text
	 *
	 * @access private
	 */
	private function reverseNumerics($text)
	{
		return preg_replace_callback(
					'/\b[0-9\.,:]+/',
					create_function('$matches', 'return strrev($matches[0]);'),
					$text
				);
	}
	
	// }}}
	
}

// }}}

?>